#!/usr/bin/env python

from __future__ import print_function
import pickle
import os.path
#imports needed for pulling emails from google api
from googleapiclient.discovery import build
from google_auth_oauthlib.flow import InstalledAppFlow
from google.auth.transport.requests import Request
#imports needed for email
from datetime import datetime, timedelta
import base64
import email
#imports needed for text parsing
from bs4 import BeautifulSoup
import re
from re import search
#imports needed for iterating through lists
import itertools

# If modifying these scopes, delete the file token.pickle.
SCOPES = ['https://www.googleapis.com/auth/gmail.readonly']

"""List all Messages of the user's mailbox matching the query."""

def ListMessagesMatchingQuery(service, user_id, query=''):
    """
    Args:
    service: Authorized Gmail API service instance.
    user_id: User's email address. The special value "me"
    can be used to indicate the authenticated user.
    query: String used to filter messages returned.
    Eg.- 'from:user@some_domain.com' for Messages from a particular sender.

    Returns:
    List of Messages that match the criteria of the query. Note that the
    returned list contains Message IDs, you must use get with the
    appropriate ID to get the details of a Message.
    """
    response = service.users().messages().list(userId=user_id,
                                           q=query).execute()
    messages = []
    if 'messages' in response:
        messages.extend(response['messages'])

    while 'nextPageToken' in response:
        page_token = response['nextPageToken']
        response = service.users().messages().list(userId=user_id, q=query,
                                         pageToken=page_token).execute()
        messages.extend(response['messages'])
   
    print("We scanned your email for tracking numbers. There were " + str(len(messages)) + " messages that matched the query string " + query)
    final_list = []
    for message in messages:
        final_list.append(message["id"])
    return final_list

"""Get Message with given ID.
"""

def GetMessageDate(service, user_id, msg_id):
    """Get a Message with given ID.

    Args:
    service: Authorized Gmail API service instance.
    user_id: User's email address. The special value "me"
    can be used to indicate the authenticated user.
    msg_id: The ID of the Message required.

    Returns:
    The date that a message was sent
    """
    message = service.users().messages().get(userId=user_id, id=msg_id).execute()
    internalDatestr = message["internalDate"]
    date = int(internalDatestr) / 1000
    finaldate = datetime.fromtimestamp(date).strftime("%Y/%m/%d")
    #print("The date of this email is: " + finaldate)
    return finaldate

def GetMimeMessage(service, user_id, msg_id):
    """Get a Message and use it to create a MIME Message.

    Args:
    service: Authorized Gmail API service instance.
    user_id: User's email address. The special value "me"
    can be used to indicate the authenticated user.
    msg_id: The ID of the Message required.

    Returns:
    A MIME Message, consisting of data from Message.
    """
    
    message = service.users().messages().get(userId=user_id, id=msg_id, format='raw').execute()
    msg_str = base64.urlsafe_b64decode(message['raw'].encode('ASCII'))
    mime_msg = email.message_from_bytes(msg_str)
    return(mime_msg)
""" Parse Message to get Tracking Number """

def ParseRegMessage(msg_id, message):
    payload = message['payload']
    body = payload['body']
    try: 
        print(body)
    except:
        print('regular message body empty')

def ParseMimeMessage(msg_id, message):
    plaintext = ""
    #hard coding in regex expressions for common usps, ups, fedex, dhl tracking #s 
    regex = ["^(94)[0-9]{20}$", "^(92)[0-9]{20}$", "^[0-9]{20}$", "^(1Z)[0-9A-Z]{16}$", 
            "^[0-9]{9}$", "^[0-9]{26}$", "^[0-9]{15}$", "^[0-9]{12}$", "^[0-9]{22}$",
            "^(EC)[0-9]{9}(US)$", "^[0-9]{10}$"]
    for part in message.walk():                                                                    
        if (part.get_content_type() == "text/plain"):
            text = part.get_payload()
            for word in text.split():
                for expr in regex:
                    if re.match(expr, word):
                        plaintext=word
                        #print(word)
                        continue
        if (part.get_content_type() == "text/html"):
            ''' parses all of the html into plaintext using beautiful soup '''
            soup = BeautifulSoup(part.get_payload(), features="lxml")
            text = soup.get_text()
            for word in text.split():
                for expr in regex:
                    if re.match(expr, word):
                        plaintext=word
                        #print(word)

    return plaintext

def main():
    """Shows basic usage of the Gmail API.
    """
    creds = None
    # The file token.pickle stores the user's access and refresh tokens, and is
    # created automatically when the authorization flow completes for the first
    # time.
    if os.path.exists('token.pickle'):
        with open('token.pickle', 'rb') as token:
            creds = pickle.load(token)
    # If there are no (valid) credentials available, let the user log in.
    if not creds or not creds.valid:
        if creds and creds.expired and creds.refresh_token:
            creds.refresh(Request())
        else:
            flow = InstalledAppFlow.from_client_secrets_file(
                'credentials.json', SCOPES)
            creds = flow.run_local_server(port=0)
        # Save the credentials for the next run
        with open('token.pickle', 'wb') as token:
            pickle.dump(creds, token)

    service = build('gmail', 'v1', credentials=creds)
    #get list of messages that mention the phrase tracking number
    messageList = ListMessagesMatchingQuery(service, "me", query="tracking number")

    outputarray = []
    datearray = []

    #iterating through messages in list to filter
    for messageid in messageList:
        # print(messageid)
        date = GetMessageDate(service, "me", messageid)
        datearray.append(date)
        message = GetMimeMessage(service, "me", messageid)
        output = ParseMimeMessage(messageid, message)
        if output != "":
            outputarray.append(output)
        else:
            #add placeholder value (only used to make arrays
            #the same length for iteration below)
            outputarray.append(".")
    #writing to a new output file
    with open ('tracking_from_email.txt', 'w') as filehandle:                                               
        prev = ""
        for (item, date) in zip(outputarray, datearray):
            timeframe = datetime.today() - timedelta(days=28)
            date = datetime.strptime(date, "%Y/%m/%d")
            if (timeframe < date):
                if ((item != prev) & (item != ".")):
                    filehandle.write('%s\n' % item)
            prev = item

if __name__ == '__main__':
   main()



#!/usr/bin/env python

import pandas as pd
import numpy as np
import csv
import os 
import re

trackinglist = []
files = os.listdir(".")
for file in files:
    if file.endswith(".txt"):
        filename = file.split(".")[0]
        regex = ["^(94)[0-9]{20}$", "^(92)[0-9]{20}$", "^[0-9]{20}$", "^(1Z)[0-9A-Z]{16}$", 
            "^[0-9]{9}$", "^[0-9]{26}$", "^[0-9]{15}$", "^[0-9]{12}$", "^[0-9]{22}$",
            "^(EC)[0-9]{9}(US)$", "^[0-9]{10}$"]
        for expr in regex:     
            if re.match(expr, filename):
                trackinglist.append(filename)

''' checking existing tracking numbers ''' 

if (trackinglist):
    print("Here are the current packages that you have on the way: ")
    for file in trackinglist:
        print(file)

if not trackinglist:
    print("You have no current packages that are on the way.")

''' checking tracking numbers found from parsing emails ''' 

emailtrackinglist = []
try: 
    with open("tracking_from_email.txt", "r") as fileHandle:
        for line in fileHandle:  
            current = line[:-1]  
            emailtrackinglist.append(current)
    if (emailtrackinglist):
        print("We found the following tracking numbers from your email: ")
        for number in emailtrackinglist:
            print(number)
        include = input("Would you like to add any of these tracking numbers to PackageTracker? [y / n] ")
        if (include == "y"):
            for number in emailtrackinglist:
                print(number)
                add = input("Add this tracking number? [y / n] ")
                if (add == "y"):
                    trackinglist.append(number)
except:
    """do nothing"""
    pass



more = False

new = input("Would you like to enter a tracking number? [y / n] ")
if (new == "y"):
    more = True

while (more):
    trackingNumber = input("Please input the tracking number of a package you want to track: ")
    trackinglist.append(trackingNumber)
    next = input("Would you like to enter another tracking number? [y / n] ")
    if (next == "n"):
       more = False

#writing list to txt file
with open ('trackingnumbers.txt', 'w') as filehandle:
    for item in trackinglist:
        filehandle.write('%s\n' % item)




#!/usr/bin/env python

#combining both scripts
#to run from one place easily
#author: ian macleod

from pyfiglet import Figlet
f = Figlet(font="slant")

print(f.renderText("packagetracker"))

import getemails
import inputPackage
import tracker

# print("Thanks for using our basic package tracker!")



#!/usr/bin/env python

from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support.expected_conditions import presence_of_element_located
import time
import os

trackinglist = []
finaltrackinglist = []
with open("trackingnumbers.txt", "r") as fileHandle:
    for line in fileHandle:
        current = line[:-1]
        trackinglist.append(current)

for number in trackinglist: 
    # creating boolean flag to track if package was delivered
    delivered = False
    
    #printing tracking number
    print(number)

    #selenium
    #runs in background 
    fireFoxOptions = webdriver.FirefoxOptions()
    fireFoxOptions.set_headless()
    with webdriver.Firefox(firefox_options=fireFoxOptions) as driver:
        driver.get("https://www.packagetrackr.com")
        #print(driver.title)
        #importing tracking list from csv

        search = driver.find_element_by_name("n")
        search.send_keys(number)
        search.send_keys(Keys.ENTER)

        time.sleep(5)
        #waiting for new page to load
        results = driver.find_elements_by_class_name("media-body")
        for result in results:
        # trying to remove listed number if item was delivered
            if ("Delivered" in result.text):
                delivered = True
    
        # writing contents to new temp file
        with open ('tempfile.txt', 'w') as filehandle:
            for result in results:
                if ("WE KNOW WHERE YOUR STUFF IS." in result.text):
                    continue
                filehandle.write('%s\n' % result.text)

        # comparing the results of old and new files if old file exists
        # returning difference to provide most accurate (recent) tracking info
        try: 
            oldfile = open((number)+'.txt', "r+")
            tempfile = open("tempfile.txt", "r+")
            old_dict = oldfile.readlines()
            new_dict = tempfile.readlines()
            oldfile.close()
            tempfile.close()
            #finding the difference between the two files
            diff = [ x for x in new_dict if x not in old_dict ]
            if diff: 
                print(diff[0].rstrip())
                old_dict = new_dict
                with open ((number)+'.txt', "w") as filehandle:
                    for line in old_dict:
                        filehandle.write('%s\n' % line.rstrip())
        # if older version of the file isn't found, create it based on temp file
        except:
            with open((number)+'.txt', "w") as filehandle:
                for result in results: 
                    if ("WE KNOW WHERE YOUR STUFF IS." in result.text):
                        continue
                    filehandle.write('%s\n' % result.text)

    if (delivered):
        print("package " + number + " was delivered, tracking number removed from list")
        try:
            os.remove((number)+'.txt')
            continue
        except:
            continue

    finaltrackinglist.append(number)

#updating tracking list txt file
with open ('trackingnumbers.txt', 'w') as filehandle:
    for item in finaltrackinglist:
        filehandle.write('%s\n' % item)

#removing tempfile 
try: 
    os.remove("tempfile.txt")
    os.remove("tracking_from_email.txt")
except: 
    pass 
#print("program complete!")



